from typing import Optional

from kognitos.bdk.runtime.client.expression import (BinaryExpression,
                                                    Expression,
                                                    UnaryExpression)
from kognitos.bdk.runtime.client.noun_phrase import NounPhrases
# pylint: disable=no-name-in-module
from kognitos.bdk.runtime.client.proto.bdk.v1.types.expression_pb2 import \
    BinaryExpression as ProtoBinaryExpression
from kognitos.bdk.runtime.client.proto.bdk.v1.types.expression_pb2 import \
    BinaryOperator as ProtoBinaryOperator
from kognitos.bdk.runtime.client.proto.bdk.v1.types.expression_pb2 import \
    Expression as ProtoExpression
from kognitos.bdk.runtime.client.proto.bdk.v1.types.expression_pb2 import \
    UnaryExpression as ProtoUnaryExpression
from kognitos.bdk.runtime.client.proto.bdk.v1.types.expression_pb2 import \
    UnaryOperator as ProtoUnaryOperator
from kognitos.bdk.runtime.client.value import Value

from . import noun_phrase as nps_mapper
from . import value as value_mapper


def map_expression(expression: Optional[Expression]) -> Optional[ProtoExpression]:
    def _map_expression_recursive(_expression: Expression) -> ProtoExpression:
        if isinstance(_expression, BinaryExpression):
            return ProtoExpression(
                binary_expression=ProtoBinaryExpression(
                    operator=ProtoBinaryOperator.ValueType(_expression.operator.value),
                    left=_map_expression_recursive(_expression.left),
                    right=_map_expression_recursive(_expression.right),
                )
            )

        if isinstance(_expression, UnaryExpression):
            return ProtoExpression(
                unary_expression=ProtoUnaryExpression(
                    operator=ProtoUnaryOperator.ValueType(_expression.operator.value),
                    inner=_map_expression_recursive(_expression.inner),
                )
            )

        # NOTE: Doing Value.__args__ in order to be compatible with python3.9. In newer versions of python
        # we can use isinstance(_expression, Value) directly.
        if isinstance(_expression, Value.__args__):
            # fmt: off
            return ProtoExpression(
                value_expression=value_mapper.map_value(_expression) # pyright: ignore [reportArgumentType]
            )
            # fmt: on

        if isinstance(_expression, NounPhrases):
            return ProtoExpression(
                noun_phrases_expression=nps_mapper.map_noun_phrases(_expression)
            )

        raise ValueError(f"Unknown expression type: {_expression}")

    if expression:
        return _map_expression_recursive(expression)
    return None



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v1.types import credential_descriptor_pb2 as bdk_dot_v1_dot_types_dot_credential__descriptor__pb2
from ....bdk.v1.types import noun_phrase_pb2 as bdk_dot_v1_dot_types_dot_noun__phrase__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8bdk/v1/types/book_custom_authentication_descriptor.proto\x12\x06bdk.v1\x1a(bdk/v1/types/credential_descriptor.proto\x1a\x1ebdk/v1/types/noun_phrase.proto"\xf4\x01\n"BookCustomAuthenticationDescriptor\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12>\n\x0bcredentials\x18\x02 \x03(\x0b2\x1c.bdk.v1.CredentialDescriptorR\x0bcredentials\x12 \n\x0bdescription\x18\x03 \x01(\tR\x0bdescription\x12\x12\n\x04name\x18\x04 \x01(\tR\x04name\x128\n\x0bnoun_phrase\x18\x05 \x01(\x0b2\x12.bdk.v1.NounPhraseH\x00R\nnounPhrase\x88\x01\x01B\x0e\n\x0c_noun_phraseB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.types.book_custom_authentication_descriptor_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_BOOKCUSTOMAUTHENTICATIONDESCRIPTOR']._serialized_start = 143
    _globals['_BOOKCUSTOMAUTHENTICATIONDESCRIPTOR']._serialized_end = 387


# pylint: disable=no-name-in-module
from kognitos.bdk.runtime.client.proto.bdk.v1.types.book_descriptor_pb2 import \
    BookDescriptor as ProtoBookDescriptor

from ..book_descriptor import BookDescriptor
from ..noun_phrase import NounPhrase
from . import book_authentication_descriptor as bad
from . import concept_descriptor as cd


def format_name_as_display_name(name: str) -> str:
    # NOTE: This is a temporary fix to ensure that the display name is formatted correctly.
    # We may need to keep this for older books, so we don't break compatibility.
    # We need to enhance this upstreams anyways, and keep the logic in sync.
    if (
        name.lower() != name
    ):  # if the name has ANY capital letters, we assume it's already formatted correctly
        return name
    splitted_name = name.split(" ")
    capitalized_words = [word.capitalize() for word in splitted_name]
    return " ".join(capitalized_words)


def map_book_descriptor(data: ProtoBookDescriptor) -> BookDescriptor:
    noun_phrase = bad.map_noun_phrase(data.noun_phrase)
    if not str(noun_phrase):
        noun_phrase = NounPhrase(data.name)
    return BookDescriptor(
        id=data.id,
        name=data.name,
        short_description=data.short_description,
        long_description=data.long_description,
        author=data.author,
        icon=data.icon,
        version=data.version,
        authentications=[
            bad.map_book_authentication_descriptor(a) for a in data.authentications
        ],
        configurations=[cd.map_concept_descriptor(c) for c in data.configurations],
        display_name=format_name_as_display_name(data.display_name or data.name),
        endpoint=data.endpoint,
        # connection_required is optional in proto, so that we support older versions of the runtime
        # that don't have this field, so we default to False.
        connection_required=data.connection_required or False,
        tags=list(data.tags),
        # discover_capable is optional in proto, so that we support older versions of the runtime
        # that don't have this field, so we default to False.
        discover_capable=data.discover_capable or False,
        noun_phrase=noun_phrase,
    )



from typing import Iterable, List

from ..concept_value import ConceptValue
from ..proto.bdk.v1.types.concept_value_pb2 import \
    ConceptValue as ProtoConceptValue  # pylint: disable=no-name-in-module
from . import noun_phrase as noun_phrase_mapper
from . import value as value_mapper


def map_concept_value(value: ProtoConceptValue) -> ConceptValue:
    return ConceptValue(
        noun_phrases=noun_phrase_mapper.map_noun_phrases(value.noun_phrases),
        value=value_mapper.map_value(value.value),
    )


def map_concept_values(values: Iterable[ProtoConceptValue]) -> List[ConceptValue]:
    return list(map(map_concept_value, values))



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v1.types import noun_phrase_pb2 as bdk_dot_v1_dot_types_dot_noun__phrase__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+bdk/v1/types/book_procedure_signature.proto\x12\x06bdk.v1\x1a\x1ebdk/v1/types/noun_phrase.proto"\xed\x02\n\x16BookProcedureSignature\x12\x18\n\x07english\x18\x01 \x01(\tR\x07english\x12\x14\n\x05verbs\x18\x02 \x03(\tR\x05verbs\x120\n\x06object\x18\x03 \x01(\x0b2\x13.bdk.v1.NounPhrasesH\x00R\x06object\x88\x01\x01\x12%\n\x0bpreposition\x18\x04 \x01(\tH\x01R\x0bpreposition\x88\x01\x01\x120\n\x06target\x18\x05 \x01(\x0b2\x13.bdk.v1.NounPhrasesH\x02R\x06target\x88\x01\x01\x12-\n\x07outputs\x18\x06 \x03(\x0b2\x13.bdk.v1.NounPhrasesR\x07outputs\x12 \n\x0cis_read_only\x18\x07 \x01(\x08R\nisReadOnly\x12!\n\x0cproper_nouns\x18\x08 \x03(\tR\x0bproperNounsB\t\n\x07_objectB\x0e\n\x0c_prepositionB\t\n\x07_targetB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.types.book_procedure_signature_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_BOOKPROCEDURESIGNATURE']._serialized_start = 88
    _globals['_BOOKPROCEDURESIGNATURE']._serialized_end = 453


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v1.types import credential_type_pb2 as bdk_dot_v1_dot_types_dot_credential__type__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(bdk/v1/types/credential_descriptor.proto\x12\x06bdk.v1\x1a"bdk/v1/types/credential_type.proto"\xc8\x01\n\x14CredentialDescriptor\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12*\n\x04type\x18\x02 \x01(\x0e2\x16.bdk.v1.CredentialTypeR\x04type\x12\x19\n\x05label\x18\x03 \x01(\tH\x00R\x05label\x88\x01\x01\x12\x18\n\x07visible\x18\x04 \x01(\x08R\x07visible\x12%\n\x0bdescription\x18\x05 \x01(\tH\x01R\x0bdescription\x88\x01\x01B\x08\n\x06_labelB\x0e\n\x0c_descriptionB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.types.credential_descriptor_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_CREDENTIALDESCRIPTOR']._serialized_start = 89
    _globals['_CREDENTIALDESCRIPTOR']._serialized_end = 289


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bgrpc/health/v1/health.proto\x12\x0egrpc.health.v1".\n\x12HealthCheckRequest\x12\x18\n\x07service\x18\x01 \x01(\tR\x07service"\xb1\x01\n\x13HealthCheckResponse\x12I\n\x06status\x18\x01 \x01(\x0e21.grpc.health.v1.HealthCheckResponse.ServingStatusR\x06status"O\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x12\x13\n\x0fSERVICE_UNKNOWN\x10\x03"\x13\n\x11HealthListRequest"\xc4\x01\n\x12HealthListResponse\x12L\n\x08statuses\x18\x01 \x03(\x0b20.grpc.health.v1.HealthListResponse.StatusesEntryR\x08statuses\x1a`\n\rStatusesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x129\n\x05value\x18\x02 \x01(\x0b2#.grpc.health.v1.HealthCheckResponseR\x05value:\x028\x012\xfd\x01\n\x06Health\x12P\n\x05Check\x12".grpc.health.v1.HealthCheckRequest\x1a#.grpc.health.v1.HealthCheckResponse\x12M\n\x04List\x12!.grpc.health.v1.HealthListRequest\x1a".grpc.health.v1.HealthListResponse\x12R\n\x05Watch\x12".grpc.health.v1.HealthCheckRequest\x1a#.grpc.health.v1.HealthCheckResponse0\x01Bp\n\x11io.grpc.health.v1B\x0bHealthProtoP\x01Z,google.golang.org/grpc/health/grpc_health_v1\xa2\x02\x0cGrpcHealthV1\xaa\x02\x0eGrpc.Health.V1b\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc.health.v1.health_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'\n\x11io.grpc.health.v1B\x0bHealthProtoP\x01Z,google.golang.org/grpc/health/grpc_health_v1\xa2\x02\x0cGrpcHealthV1\xaa\x02\x0eGrpc.Health.V1'
    _globals['_HEALTHLISTRESPONSE_STATUSESENTRY']._options = None
    _globals['_HEALTHLISTRESPONSE_STATUSESENTRY']._serialized_options = b'8\x01'
    _globals['_HEALTHCHECKREQUEST']._serialized_start = 47
    _globals['_HEALTHCHECKREQUEST']._serialized_end = 93
    _globals['_HEALTHCHECKRESPONSE']._serialized_start = 96
    _globals['_HEALTHCHECKRESPONSE']._serialized_end = 273
    _globals['_HEALTHCHECKRESPONSE_SERVINGSTATUS']._serialized_start = 194
    _globals['_HEALTHCHECKRESPONSE_SERVINGSTATUS']._serialized_end = 273
    _globals['_HEALTHLISTREQUEST']._serialized_start = 275
    _globals['_HEALTHLISTREQUEST']._serialized_end = 294
    _globals['_HEALTHLISTRESPONSE']._serialized_start = 297
    _globals['_HEALTHLISTRESPONSE']._serialized_end = 493
    _globals['_HEALTHLISTRESPONSE_STATUSESENTRY']._serialized_start = 397
    _globals['_HEALTHLISTRESPONSE_STATUSESENTRY']._serialized_end = 493
    _globals['_HEALTH']._serialized_start = 496
    _globals['_HEALTH']._serialized_end = 749


import datetime
from dataclasses import dataclass, field
from typing import Any, Dict, List, Union

from kognitos.bdk.runtime.client.noun_phrase import \
    NounPhrase  # pylint: disable=no-name-in-module
from kognitos.bdk.runtime.client.sensitive import \
    Sensitive  # pylint: disable=no-name-in-module

from .base import BdkDescriptorBase

NoneType = type(None)


@dataclass
class RemoteFile(BdkDescriptorBase):
    url: str

    def human_readable(self) -> str:
        """
        Returns a string representation of the RemoteFile.
        """
        return "RemoteFile"


@dataclass
class File(BdkDescriptorBase):
    filename: str
    content: bytes

    def human_readable(self) -> str:
        """
        Returns a string representation of the File.
        """
        return "File"


FileValue = Union[RemoteFile, File]


@dataclass
class DictionaryValue(BdkDescriptorBase):
    value: Dict[str, "Value"]
    is_a: List[NounPhrase] = field(default_factory=list)

    def human_readable(self) -> str:
        """
        Returns a string representation of the DictionaryValue.
        """
        str_is_a = [str(is_a) for is_a in self.is_a]
        return f"Object({', '.join(str_is_a)})"

    def as_dict(self) -> Dict[str, Any]:
        """
        Returns the DictionaryValue as a native Python dictionary.
        """

        def _convert_value(value: "Value") -> Any:

            if isinstance(value, DictionaryValue):
                return value.as_dict()
            if isinstance(value, list):
                return [_convert_value(v) for v in value]

            return value

        return {k: _convert_value(v) for k, v in self.value.items()}


@dataclass
class OpaqueValue(BdkDescriptorBase):
    value: bytes
    is_a: List[NounPhrase] = field(default_factory=list)

    def human_readable(self) -> str:
        """
        Returns a string representation of the OpaqueValue.
        """
        # turn into comma separated string
        str_is_a = [str(is_a) for is_a in self.is_a]
        return f"Object({', '.join(str_is_a)})"


@dataclass
class RemoteTable(BdkDescriptorBase):
    url: str

    def human_readable(self) -> str:
        """
        Returns a string representation of the RemoteTable.
        """
        return "RemoteTable"


@dataclass
class Column:
    name: str
    values: List["Value"]

    def human_readable(self) -> str:
        """
        Returns a string representation of the Column.
        """
        return f"Column(name={self.name})"


@dataclass
class Table(BdkDescriptorBase):
    columns: List[Column]

    def human_readable(self) -> str:
        """
        Returns a string representation of a Table.
        """
        column_names = [column.name for column in self.columns]
        column_names_str = ", ".join(column_names)
        return f"Table with columns: {column_names_str}"


TableValue = Union[Table, RemoteTable]


NullValue = type(None)
TextValue = str
NumberValue = float
BooleanValue = bool
ConceptualValue = NounPhrase
DatetimeValue = datetime.datetime
DateValue = datetime.date
TimeValue = datetime.time
ListValue = List
SensitiveValue = Sensitive

Value = Union[
    ConceptualValue,
    NumberValue,
    NullValue,
    TextValue,
    BooleanValue,
    FileValue,
    OpaqueValue,
    DictionaryValue,
    ListValue,
    DatetimeValue,
    DateValue,
    TimeValue,
    TableValue,
    SensitiveValue,
]



from bdk.v2.types import noun_phrase_pb2 as _noun_phrase_pb2
from bdk.v2.types import value_pb2 as _value_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class AnsweredQuestion(_message.Message):
    __slots__ = ('noun_phrases', 'answer')
    NOUN_PHRASES_FIELD_NUMBER: _ClassVar[int]
    ANSWER_FIELD_NUMBER: _ClassVar[int]
    noun_phrases: _noun_phrase_pb2.NounPhrases
    answer: _value_pb2.Value

    def __init__(self, noun_phrases: _Optional[_Union[_noun_phrase_pb2.NounPhrases, _Mapping]]=..., answer: _Optional[_Union[_value_pb2.Value, _Mapping]]=...) -> None:
        ...


import base64
import json
from builtins import bool, bytes, dict, float, int, list, str
from datetime import date, datetime, time
from enum import Enum
from typing import Any, List, Optional, Type

# NOTE: The following list of imports are here so that they're available in the global scope and we're
# able to serialize/deserialize them properly.
# pylint: disable=unused-import
from kognitos.bdk.runtime.client.book_authentication_descriptor import (
    BookAuthenticationDescriptor, BookCustomAuthenticationDescriptor,
    BookOAuthAuthenticationDescriptor, OauthArgumentDescriptor, OauthFlow,
    OauthProvider)
from kognitos.bdk.runtime.client.book_descriptor import BookDescriptor
from kognitos.bdk.runtime.client.book_procedure_descriptor import \
    BookProcedureDescriptor
from kognitos.bdk.runtime.client.book_procedure_signature import \
    BookProcedureSignature
from kognitos.bdk.runtime.client.concept_descriptor import ConceptDescriptor
from kognitos.bdk.runtime.client.concept_type import (
    ConceptAnyType, ConceptDictionaryType, ConceptDictionaryTypeField,
    ConceptEnumType, ConceptEnumTypeMember, ConceptListType, ConceptOpaqueType,
    ConceptOptionalType, ConceptScalarType, ConceptSelfType,
    ConceptSensitiveType, ConceptTableType, ConceptTableTypeColumn,
    ConceptUnionType)
from kognitos.bdk.runtime.client.concept_value import ConceptValue
from kognitos.bdk.runtime.client.connection_required import ConnectionRequired
from kognitos.bdk.runtime.client.credential_descriptor import (
    CredentialDescriptor, CredentialType)
from kognitos.bdk.runtime.client.environment_information import \
    EnvironmentInformation
from kognitos.bdk.runtime.client.example_descriptor import ExampleDescriptor
from kognitos.bdk.runtime.client.expression import (BinaryExpression,
                                                    BinaryOperator,
                                                    UnaryExpression)
from kognitos.bdk.runtime.client.input_concept import InputConcept
from kognitos.bdk.runtime.client.noun_phrase import NounPhrase, NounPhrases
from kognitos.bdk.runtime.client.promise import Promise
from kognitos.bdk.runtime.client.question_answer import (AnsweredQuestion,
                                                         Question)
from kognitos.bdk.runtime.client.question_descriptor import QuestionDescriptor
from kognitos.bdk.runtime.client.sensitive import Sensitive
from kognitos.bdk.runtime.client.test_connection import TestConnectionResponse
from kognitos.bdk.runtime.client.value import (BooleanValue, ConceptualValue,
                                               DatetimeValue, DateValue,
                                               DictionaryValue, File,
                                               ListValue, NullValue,
                                               NumberValue, OpaqueValue,
                                               RemoteFile, TextValue,
                                               TimeValue)

# pylint: enable=unused-import


class BDKSerializableNone:
    """Python 3.9 doesn't have NoneType withi `typing`, so we need to create our own for serialization's sake"""


def to_bytes(obj: Any) -> bytes:
    """
    Creates a representation of the object as a dictionary of

    key: [type, serialized_value]
    """

    def to_dict(obj):
        if isinstance(obj, (str, int, float, bool, bytes)):
            if isinstance(obj, bytes):
                return (obj.__class__.__name__, obj.decode("utf-8"))
            return (obj.__class__.__name__, obj)

        if hasattr(obj, "__dataclass_fields__"):
            as_dict = obj.__dict__
            return (obj.__class__.__name__, {k: to_dict(v) for k, v in as_dict.items()})

        if isinstance(obj, dict):
            return (obj.__class__.__name__, {k: to_dict(v) for k, v in obj.items()})

        if isinstance(obj, list):
            return (obj.__class__.__name__, [to_dict(v) for v in obj])

        if isinstance(obj, Enum):
            return (obj.__class__.__name__, obj.value)

        if obj is None:
            return ("BDKSerializableNone", None)

        if isinstance(obj, (date, datetime, time)):
            return (obj.__class__.__name__, obj.isoformat())

        raise ValueError(f"Unsupported type: {type(obj)}")

    return bytes(json.dumps(to_dict(obj)), "utf-8")


def from_bytes(obj: bytes, additional_types: Optional[List[Type]] = None) -> Any:
    as_dict = json.loads(obj.decode("utf-8"))
    additional_types_map = (
        {t.__name__: t for t in additional_types} if additional_types else {}
    )

    def from_dict(as_dict):
        type_name, value = as_dict

        is_valid_type = type_name in globals() or type_name in additional_types_map

        if not is_valid_type:
            raise ValueError(f"Unsupported type: {type_name}")

        cls = globals().get(type_name) or additional_types_map.get(type_name)

        if not cls:
            raise ValueError(f"Unsupported type: {type_name}")

        if cls in (str, int, float, bool):
            return cls(value)

        if cls == bytes:
            return bytes(value, "utf-8")

        if hasattr(cls, "__dataclass_fields__"):
            return cls(**{k: from_dict(v) for k, v in value.items()})

        if cls == dict:
            return cls({k: from_dict(v) for k, v in value.items()})

        if cls == list:
            return [from_dict(v) for v in value]

        if issubclass(cls, Enum):
            return cls(value)

        if cls == BDKSerializableNone:
            return None

        if cls == date:
            return date.fromisoformat(value)

        if cls == datetime:
            return datetime.fromisoformat(value)

        if cls == time:
            return time.fromisoformat(value)

        raise ValueError(f"Unsupported type: {type(obj)}")

    return from_dict(as_dict)


def compute_hash(obj: Any) -> bytes:
    return base64.b64encode(to_bytes(obj))



from bdk.v1.types import book_authentication_descriptor_pb2 as _book_authentication_descriptor_pb2
from bdk.v1.types import book_procedure_descriptor_pb2 as _book_procedure_descriptor_pb2
from bdk.v1.types import concept_descriptor_pb2 as _concept_descriptor_pb2
from bdk.v1.types import noun_phrase_pb2 as _noun_phrase_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class BookDescriptor(_message.Message):
    __slots__ = ('id', 'name', 'short_description', 'long_description', 'author', 'icon', 'version', 'authentications', 'configurations', 'display_name', 'endpoint', 'connection_required', 'discover_capable', 'tags', 'procedures', 'noun_phrase')
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    SHORT_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    LONG_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    AUTHOR_FIELD_NUMBER: _ClassVar[int]
    ICON_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    AUTHENTICATIONS_FIELD_NUMBER: _ClassVar[int]
    CONFIGURATIONS_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_REQUIRED_FIELD_NUMBER: _ClassVar[int]
    DISCOVER_CAPABLE_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    PROCEDURES_FIELD_NUMBER: _ClassVar[int]
    NOUN_PHRASE_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    short_description: str
    long_description: str
    author: str
    icon: bytes
    version: str
    authentications: _containers.RepeatedCompositeFieldContainer[_book_authentication_descriptor_pb2.BookAuthenticationDescriptor]
    configurations: _containers.RepeatedCompositeFieldContainer[_concept_descriptor_pb2.ConceptDescriptor]
    display_name: str
    endpoint: str
    connection_required: bool
    discover_capable: bool
    tags: _containers.RepeatedScalarFieldContainer[str]
    procedures: _containers.RepeatedCompositeFieldContainer[_book_procedure_descriptor_pb2.BookProcedureDescriptorV2]
    noun_phrase: _noun_phrase_pb2.NounPhrase

    def __init__(self, id: _Optional[str]=..., name: _Optional[str]=..., short_description: _Optional[str]=..., long_description: _Optional[str]=..., author: _Optional[str]=..., icon: _Optional[bytes]=..., version: _Optional[str]=..., authentications: _Optional[_Iterable[_Union[_book_authentication_descriptor_pb2.BookAuthenticationDescriptor, _Mapping]]]=..., configurations: _Optional[_Iterable[_Union[_concept_descriptor_pb2.ConceptDescriptor, _Mapping]]]=..., display_name: _Optional[str]=..., endpoint: _Optional[str]=..., connection_required: bool=..., discover_capable: bool=..., tags: _Optional[_Iterable[str]]=..., procedures: _Optional[_Iterable[_Union[_book_procedure_descriptor_pb2.BookProcedureDescriptorV2, _Mapping]]]=..., noun_phrase: _Optional[_Union[_noun_phrase_pb2.NounPhrase, _Mapping]]=...) -> None:
        ...


from bdk.v1.types import question_pb2 as _question_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class QuestionResponse(_message.Message):
    __slots__ = ('questions',)
    QUESTIONS_FIELD_NUMBER: _ClassVar[int]
    questions: _containers.RepeatedCompositeFieldContainer[_question_pb2.Question]

    def __init__(self, questions: _Optional[_Iterable[_Union[_question_pb2.Question, _Mapping]]]=...) -> None:
        ...


from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional
DESCRIPTOR: _descriptor.FileDescriptor

class Context(_message.Message):
    __slots__ = ('trace_id', 'span_id', 'worker_id', 'department_id', 'knowledge_id', 'line_id')
    TRACE_ID_FIELD_NUMBER: _ClassVar[int]
    SPAN_ID_FIELD_NUMBER: _ClassVar[int]
    WORKER_ID_FIELD_NUMBER: _ClassVar[int]
    DEPARTMENT_ID_FIELD_NUMBER: _ClassVar[int]
    KNOWLEDGE_ID_FIELD_NUMBER: _ClassVar[int]
    LINE_ID_FIELD_NUMBER: _ClassVar[int]
    trace_id: int
    span_id: int
    worker_id: str
    department_id: str
    knowledge_id: str
    line_id: str

    def __init__(self, trace_id: _Optional[int]=..., span_id: _Optional[int]=..., worker_id: _Optional[str]=..., department_id: _Optional[str]=..., knowledge_id: _Optional[str]=..., line_id: _Optional[str]=...) -> None:
        ...


from kognitos.bdk.runtime.client.book_authentication_descriptor import \
    OauthArgumentDescriptor
# pylint: disable=no-name-in-module
from kognitos.bdk.runtime.client.proto.bdk.v1.types.oauth_argument_descriptor_pb2 import \
    OAuthArgumentDescriptor as ProtoOAuthArgumentDescriptor

# pylint: enable=no-name-in-module


def map_oauth_argument_descriptor(
    data: ProtoOAuthArgumentDescriptor,
) -> OauthArgumentDescriptor:
    return OauthArgumentDescriptor(
        id=data.id,
        name=data.name,
        description=data.description,
    )



from bdk.v2.types import concept_type_pb2 as _concept_type_pb2
from bdk.v2.types import noun_phrase_pb2 as _noun_phrase_pb2
from bdk.v2.types import value_pb2 as _value_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class Question(_message.Message):
    __slots__ = ('noun_phrases', 'concept_type', 'choices', 'text')
    NOUN_PHRASES_FIELD_NUMBER: _ClassVar[int]
    CONCEPT_TYPE_FIELD_NUMBER: _ClassVar[int]
    CHOICES_FIELD_NUMBER: _ClassVar[int]
    TEXT_FIELD_NUMBER: _ClassVar[int]
    noun_phrases: _noun_phrase_pb2.NounPhrases
    concept_type: _concept_type_pb2.ConceptType
    choices: _containers.RepeatedCompositeFieldContainer[_value_pb2.Value]
    text: str

    def __init__(self, noun_phrases: _Optional[_Union[_noun_phrase_pb2.NounPhrases, _Mapping]]=..., concept_type: _Optional[_Union[_concept_type_pb2.ConceptType, _Mapping]]=..., choices: _Optional[_Iterable[_Union[_value_pb2.Value, _Mapping]]]=..., text: _Optional[str]=...) -> None:
        ...


from dataclasses import dataclass

from kognitos.bdk.runtime.client.noun_phrase import NounPhrases
from kognitos.bdk.runtime.client.value import Value

from .base import BdkDescriptorBase


@dataclass
class ConceptValue(BdkDescriptorBase):
    noun_phrases: NounPhrases
    value: Value



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v1.types import book_connection_descriptor_pb2 as bdk_dot_v1_dot_types_dot_book__connection__descriptor__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1bdk/v1/responses/retrieve_books_connections.proto\x12\x06bdk.v1\x1a-bdk/v1/types/book_connection_descriptor.proto"\xb0\x01\n RetrieveBooksConnectionsResponse\x12K\n\x10book_connections\x18\x01 \x03(\x0b2 .bdk.v1.BookConnectionDescriptorR\x0fbookConnections\x12+\n\x0fnext_page_token\x18\x02 \x01(\tH\x00R\rnextPageToken\x88\x01\x01B\x12\n\x10_next_page_tokenB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.responses.retrieve_books_connections_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_RETRIEVEBOOKSCONNECTIONSRESPONSE']._serialized_start = 109
    _globals['_RETRIEVEBOOKSCONNECTIONSRESPONSE']._serialized_end = 285


from bdk.v1.requests import authentication_pb2 as _authentication_pb2
from bdk.v1.requests import offload_pb2 as _offload_pb2
from bdk.v1.types import answered_question_pb2 as _answered_question_pb2
from bdk.v1.types import concept_value_pb2 as _concept_value_pb2
from bdk.v1.types import promise_pb2 as _promise_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class ResolvePromiseRequest(_message.Message):
    __slots__ = ('name', 'version', 'authentication', 'procedure_id', 'promise', 'offload', 'answered_questions', 'configurations')
    NAME_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    AUTHENTICATION_FIELD_NUMBER: _ClassVar[int]
    PROCEDURE_ID_FIELD_NUMBER: _ClassVar[int]
    PROMISE_FIELD_NUMBER: _ClassVar[int]
    OFFLOAD_FIELD_NUMBER: _ClassVar[int]
    ANSWERED_QUESTIONS_FIELD_NUMBER: _ClassVar[int]
    CONFIGURATIONS_FIELD_NUMBER: _ClassVar[int]
    name: str
    version: str
    authentication: _authentication_pb2.Authentication
    procedure_id: str
    promise: _promise_pb2.Promise
    offload: _offload_pb2.Offload
    answered_questions: _containers.RepeatedCompositeFieldContainer[_answered_question_pb2.AnsweredQuestion]
    configurations: _containers.RepeatedCompositeFieldContainer[_concept_value_pb2.ConceptValue]

    def __init__(self, name: _Optional[str]=..., version: _Optional[str]=..., authentication: _Optional[_Union[_authentication_pb2.Authentication, _Mapping]]=..., procedure_id: _Optional[str]=..., promise: _Optional[_Union[_promise_pb2.Promise, _Mapping]]=..., offload: _Optional[_Union[_offload_pb2.Offload, _Mapping]]=..., answered_questions: _Optional[_Iterable[_Union[_answered_question_pb2.AnsweredQuestion, _Mapping]]]=..., configurations: _Optional[_Iterable[_Union[_concept_value_pb2.ConceptValue, _Mapping]]]=...) -> None:
        ...


from bdk.v1.responses import promise_pb2 as _promise_pb2
from bdk.v1.responses import question_pb2 as _question_pb2
from bdk.v1.types import concept_value_pb2 as _concept_value_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class InvokeProcedureResponse(_message.Message):
    __slots__ = ('output_concepts',)
    OUTPUT_CONCEPTS_FIELD_NUMBER: _ClassVar[int]
    output_concepts: _containers.RepeatedCompositeFieldContainer[_concept_value_pb2.ConceptValue]

    def __init__(self, output_concepts: _Optional[_Iterable[_Union[_concept_value_pb2.ConceptValue, _Mapping]]]=...) -> None:
        ...

class InvokeProcedureResponseV2(_message.Message):
    __slots__ = ('response', 'question', 'promise')
    RESPONSE_FIELD_NUMBER: _ClassVar[int]
    QUESTION_FIELD_NUMBER: _ClassVar[int]
    PROMISE_FIELD_NUMBER: _ClassVar[int]
    response: InvokeProcedureResponse
    question: _question_pb2.QuestionResponse
    promise: _promise_pb2.PromiseResponse

    def __init__(self, response: _Optional[_Union[InvokeProcedureResponse, _Mapping]]=..., question: _Optional[_Union[_question_pb2.QuestionResponse, _Mapping]]=..., promise: _Optional[_Union[_promise_pb2.PromiseResponse, _Mapping]]=...) -> None:
        ...


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v2.responses import discover_procedures_pb2 as bdk_dot_v2_dot_responses_dot_discover__procedures__pb2
from ....bdk.v2.responses import environment_information_pb2 as bdk_dot_v2_dot_responses_dot_environment__information__pb2
from ....bdk.v2.responses import error_pb2 as bdk_dot_v2_dot_responses_dot_error__pb2
from ....bdk.v2.responses import invoke_procedure_pb2 as bdk_dot_v2_dot_responses_dot_invoke__procedure__pb2
from ....bdk.v2.responses import promise_pb2 as bdk_dot_v2_dot_responses_dot_promise__pb2
from ....bdk.v2.responses import question_pb2 as bdk_dot_v2_dot_responses_dot_question__pb2
from ....bdk.v2.responses import retrieve_book_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__book__pb2
from ....bdk.v2.responses import retrieve_book_procedures_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__book__procedures__pb2
from ....bdk.v2.responses import retrieve_books_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__books__pb2
from ....bdk.v2.responses import retrieve_discoverables_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__discoverables__pb2
from ....bdk.v2.responses import retrieve_tags_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__tags__pb2
from ....bdk.v2.responses import test_connection_pb2 as bdk_dot_v2_dot_responses_dot_test__connection__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fbdk/v2/responses/response.proto\x12\x06bdk.v2\x1a*bdk/v2/responses/discover_procedures.proto\x1a.bdk/v2/responses/environment_information.proto\x1a\x1cbdk/v2/responses/error.proto\x1a\'bdk/v2/responses/invoke_procedure.proto\x1a\x1ebdk/v2/responses/promise.proto\x1a\x1fbdk/v2/responses/question.proto\x1a$bdk/v2/responses/retrieve_book.proto\x1a/bdk/v2/responses/retrieve_book_procedures.proto\x1a%bdk/v2/responses/retrieve_books.proto\x1a-bdk/v2/responses/retrieve_discoverables.proto\x1a$bdk/v2/responses/retrieve_tags.proto\x1a&bdk/v2/responses/test_connection.proto"\xac\x07\n\x08Response\x12%\n\x05error\x18\x01 \x01(\x0b2\r.bdk.v2.ErrorH\x00R\x05error\x12a\n\x17environment_information\x18\x02 \x01(\x0b2&.bdk.v2.EnvironmentInformationResponseH\x00R\x16environmentInformation\x12H\n\x0fretrieved_books\x18\x03 \x01(\x0b2\x1d.bdk.v2.RetrieveBooksResponseH\x00R\x0eretrievedBooks\x12E\n\x0eretrieved_book\x18\x04 \x01(\x0b2\x1c.bdk.v2.RetrieveBookResponseH\x00R\rretrievedBook\x12[\n\x14retrieved_procedures\x18\x05 \x01(\x0b2&.bdk.v2.RetrieveBookProceduresResponseH\x00R\x13retrievedProcedures\x12M\n\x11tested_connection\x18\x06 \x01(\x0b2\x1e.bdk.v2.TestConnectionResponseH\x00R\x10testedConnection\x12N\n\x11invoked_procedure\x18\x07 \x01(\x0b2\x1f.bdk.v2.InvokeProcedureResponseH\x00R\x10invokedProcedure\x12Y\n\x15discovered_procedures\x18\x08 \x01(\x0b2".bdk.v2.DiscoverProceduresResponseH\x00R\x14discoveredProcedures\x126\n\x08question\x18\t \x01(\x0b2\x18.bdk.v2.QuestionResponseH\x00R\x08question\x12`\n\x17retrieved_discoverables\x18\n \x01(\x0b2%.bdk.v2.RetrieveDiscoverablesResponseH\x00R\x16retrievedDiscoverables\x12E\n\x0eretrieved_tags\x18\x0b \x01(\x0b2\x1c.bdk.v2.RetrieveTagsResponseH\x00R\rretrievedTags\x123\n\x07promise\x18\x0c \x01(\x0b2\x17.bdk.v2.PromiseResponseH\x00R\x07promiseB\x18\n\x16response_discriminatorB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v2.responses.response_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_RESPONSE']._serialized_start = 523
    _globals['_RESPONSE']._serialized_end = 1463


from dataclasses import dataclass
from typing import Optional


@dataclass
class RequestContext:
    trace_id: Optional[int] = None
    span_id: Optional[int] = None
    worker_id: Optional[str] = None
    department_id: Optional[str] = None
    knowledge_id: Optional[str] = None
    line_id: Optional[str] = None



from dataclasses import dataclass

from .base import BdkDescriptorBase
from .noun_phrase import NounPhrases
from .value import Value


@dataclass
class InputConcept(BdkDescriptorBase):
    noun_phrases: NounPhrases
    value: Value



from dataclasses import dataclass
from enum import Enum
from typing import Union

from .noun_phrase import NounPhrases
from .value import Value


class BinaryOperator(Enum):
    BINARY_OPERATOR_AND = 0
    BINARY_OPERATOR_OR = 1
    BINARY_OPERATOR_EQUALS = 2
    BINARY_OPERATOR_NOT_EQUALS = 3
    BINARY_OPERATOR_IN = 4
    BINARY_OPERATOR_HAS = 5
    BINARY_OPERATOR_LESS_THAN = 6
    BINARY_OPERATOR_GREATER_THAN = 7
    BINARY_OPERATOR_LESS_THAN_OR_EQUAL = 8
    BINARY_OPERATOR_GREATER_THAN_OR_EQUAL = 9


class UnaryOperator(Enum):
    UNARY_OPERATOR_NOT = 0


@dataclass
class BinaryExpression:
    operator: BinaryOperator
    left: "Expression"
    right: "Expression"


@dataclass
class UnaryExpression:
    operator: UnaryOperator
    inner: "Expression"


Expression = Union[BinaryExpression, UnaryExpression, Value, NounPhrases]



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v1.types import value_pb2 as bdk_dot_v1_dot_types_dot_value__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$bdk/v1/requests/authentication.proto\x12\x06bdk.v1\x1a\x18bdk/v1/types/value.proto"\x95\x01\n\x0eAuthentication\x12+\n\x11authentication_id\x18\x03 \x01(\tR\x10authenticationId\x12V\n\x1aauthentication_credentials\x18\x04 \x03(\x0b2\x17.bdk.v1.CredentialValueR\x19authenticationCredentials"F\n\x0fCredentialValue\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12#\n\x05value\x18\x02 \x01(\x0b2\r.bdk.v1.ValueR\x05valueB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.requests.authentication_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_AUTHENTICATION']._serialized_start = 75
    _globals['_AUTHENTICATION']._serialized_end = 224
    _globals['_CREDENTIALVALUE']._serialized_start = 226
    _globals['_CREDENTIALVALUE']._serialized_end = 296


from dataclasses import dataclass
from enum import Enum
from typing import List, Optional, Union

from kognitos.bdk.runtime.client.noun_phrase import NounPhrase

from .base import BdkDescriptorBase
from .credential_descriptor import CredentialDescriptor


@dataclass
class BookCustomAuthenticationDescriptor(BdkDescriptorBase):
    id: str
    credentials: List[CredentialDescriptor]
    description: str
    name: str
    noun_phrase: Optional[NounPhrase]


class OauthProvider(Enum):
    OAUTH_PROVIDER_MICROSOFT = 0
    OAUTH_PROVIDER_GOOGLE = 1
    OAUTH_PROVIDER_GENERIC = 2


class OauthFlow(Enum):
    OAUTH_FLOW_AUTHORIZATION_CODE = 0
    OAUTH_FLOW_CLIENT_CREDENTIALS = 1


@dataclass
class OauthArgumentDescriptor(BdkDescriptorBase):
    id: str
    name: str
    description: str


@dataclass
class BookOAuthAuthenticationDescriptor(BdkDescriptorBase):
    id: str
    provider: OauthProvider
    flows: List[OauthFlow]
    authorize_endpoint: str
    token_endpoint: str
    scope: List[str]
    name: str
    arguments: List[OauthArgumentDescriptor]


BookAuthenticationDescriptor = Union[
    BookCustomAuthenticationDescriptor, BookOAuthAuthenticationDescriptor
]



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional
DESCRIPTOR: _descriptor.FileDescriptor

class UpgradeBookConnectionResponse(_message.Message):
    __slots__ = ('connection_id',)
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    connection_id: str

    def __init__(self, connection_id: _Optional[str]=...) -> None:
        ...


from . import answered_question_pb2
from . import book_authentication_descriptor_pb2
from . import book_connection_descriptor_pb2
from . import book_custom_authentication_descriptor_pb2
from . import book_descriptor_pb2
from . import book_oauh_authentication_descriptor_pb2
from . import book_procedure_descriptor_pb2
from . import book_procedure_signature_pb2
from . import concept_descriptor_pb2
from . import concept_type_pb2
from . import concept_value_pb2
from . import config_value_pb2
from . import connection_required_pb2
from . import credential_descriptor_pb2
from . import credential_type_pb2
from . import date_pb2
from . import discoverable_pb2
from . import example_descriptor_pb2
from . import expression_pb2
from . import noun_phrase_pb2
from . import oauth_argument_descriptor_pb2
from . import oauth_flow_pb2
from . import oauth_provider_pb2
from . import promise_pb2
from . import question_descriptor_pb2
from . import question_pb2
from . import time_pb2
from . import value_pb2



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar
DESCRIPTOR: _descriptor.FileDescriptor

class EnvironmentInformationRequest(_message.Message):
    __slots__ = ()

    def __init__(self) -> None:
        ...


from typing import Optional

from kognitos.bdk.runtime.client.offload import AWSOffload, Offload
from kognitos.bdk.runtime.client.proto.bdk.v1.requests.offload_pb2 import \
    AWSOffload as ProtoAWSOffload  # pylint: disable=no-name-in-module
from kognitos.bdk.runtime.client.proto.bdk.v1.requests.offload_pb2 import \
    Offload as ProtoOffload  # pylint: disable=no-name-in-module


def map_offload(offload: Optional[Offload]) -> Optional[ProtoOffload]:
    if not offload:
        return None

    if isinstance(offload, AWSOffload):
        return ProtoOffload(
            aws=ProtoAWSOffload(
                access_key=offload.access_key,
                secret_key=offload.secret_key,
                session_token=offload.session_token,
                region=offload.region,
                bucket=offload.bucket,
                folder_name=offload.folder_name,
            )
        )

    raise ValueError(f"Unknown offload type: {offload}")



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class Http(_message.Message):
    __slots__ = ('rules', 'fully_decode_reserved_expansion')
    RULES_FIELD_NUMBER: _ClassVar[int]
    FULLY_DECODE_RESERVED_EXPANSION_FIELD_NUMBER: _ClassVar[int]
    rules: _containers.RepeatedCompositeFieldContainer[HttpRule]
    fully_decode_reserved_expansion: bool

    def __init__(self, rules: _Optional[_Iterable[_Union[HttpRule, _Mapping]]]=..., fully_decode_reserved_expansion: bool=...) -> None:
        ...

class HttpRule(_message.Message):
    __slots__ = ('selector', 'get', 'put', 'post', 'delete', 'patch', 'custom', 'body', 'response_body', 'additional_bindings')
    SELECTOR_FIELD_NUMBER: _ClassVar[int]
    GET_FIELD_NUMBER: _ClassVar[int]
    PUT_FIELD_NUMBER: _ClassVar[int]
    POST_FIELD_NUMBER: _ClassVar[int]
    DELETE_FIELD_NUMBER: _ClassVar[int]
    PATCH_FIELD_NUMBER: _ClassVar[int]
    CUSTOM_FIELD_NUMBER: _ClassVar[int]
    BODY_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_BODY_FIELD_NUMBER: _ClassVar[int]
    ADDITIONAL_BINDINGS_FIELD_NUMBER: _ClassVar[int]
    selector: str
    get: str
    put: str
    post: str
    delete: str
    patch: str
    custom: CustomHttpPattern
    body: str
    response_body: str
    additional_bindings: _containers.RepeatedCompositeFieldContainer[HttpRule]

    def __init__(self, selector: _Optional[str]=..., get: _Optional[str]=..., put: _Optional[str]=..., post: _Optional[str]=..., delete: _Optional[str]=..., patch: _Optional[str]=..., custom: _Optional[_Union[CustomHttpPattern, _Mapping]]=..., body: _Optional[str]=..., response_body: _Optional[str]=..., additional_bindings: _Optional[_Iterable[_Union[HttpRule, _Mapping]]]=...) -> None:
        ...

class CustomHttpPattern(_message.Message):
    __slots__ = ('kind', 'path')
    KIND_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    kind: str
    path: str

    def __init__(self, kind: _Optional[str]=..., path: _Optional[str]=...) -> None:
        ...


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v1.requests import context_pb2 as bdk_dot_v1_dot_requests_dot_context__pb2
from ....bdk.v1.requests import discover_procedures_pb2 as bdk_dot_v1_dot_requests_dot_discover__procedures__pb2
from ....bdk.v1.requests import environment_information_pb2 as bdk_dot_v1_dot_requests_dot_environment__information__pb2
from ....bdk.v1.requests import invoke_procedure_pb2 as bdk_dot_v1_dot_requests_dot_invoke__procedure__pb2
from ....bdk.v1.requests import resolve_promise_pb2 as bdk_dot_v1_dot_requests_dot_resolve__promise__pb2
from ....bdk.v1.requests import retrieve_book_pb2 as bdk_dot_v1_dot_requests_dot_retrieve__book__pb2
from ....bdk.v1.requests import retrieve_book_procedures_pb2 as bdk_dot_v1_dot_requests_dot_retrieve__book__procedures__pb2
from ....bdk.v1.requests import retrieve_books_pb2 as bdk_dot_v1_dot_requests_dot_retrieve__books__pb2
from ....bdk.v1.requests import retrieve_discoverables_pb2 as bdk_dot_v1_dot_requests_dot_retrieve__discoverables__pb2
from ....bdk.v1.requests import retrieve_tags_pb2 as bdk_dot_v1_dot_requests_dot_retrieve__tags__pb2
from ....bdk.v1.requests import test_connection_pb2 as bdk_dot_v1_dot_requests_dot_test__connection__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dbdk/v1/requests/request.proto\x12\x06bdk.v1\x1a\x1dbdk/v1/requests/context.proto\x1a)bdk/v1/requests/discover_procedures.proto\x1a-bdk/v1/requests/environment_information.proto\x1a&bdk/v1/requests/invoke_procedure.proto\x1a%bdk/v1/requests/resolve_promise.proto\x1a#bdk/v1/requests/retrieve_book.proto\x1a.bdk/v1/requests/retrieve_book_procedures.proto\x1a$bdk/v1/requests/retrieve_books.proto\x1a,bdk/v1/requests/retrieve_discoverables.proto\x1a#bdk/v1/requests/retrieve_tags.proto\x1a%bdk/v1/requests/test_connection.proto"\xee\x06\n\x07Request\x12)\n\x07context\x18\x01 \x01(\x0b2\x0f.bdk.v1.ContextR\x07context\x12`\n\x17environment_information\x18\x02 \x01(\x0b2%.bdk.v1.EnvironmentInformationRequestH\x00R\x16environmentInformation\x12E\n\x0eretrieve_books\x18\x03 \x01(\x0b2\x1c.bdk.v1.RetrieveBooksRequestH\x00R\rretrieveBooks\x12B\n\rretrieve_book\x18\x04 \x01(\x0b2\x1b.bdk.v1.RetrieveBookRequestH\x00R\x0cretrieveBook\x12X\n\x13retrieve_procedures\x18\x05 \x01(\x0b2%.bdk.v1.RetrieveBookProceduresRequestH\x00R\x12retrieveProcedures\x12H\n\x0ftest_connection\x18\x06 \x01(\x0b2\x1d.bdk.v1.TestConnectionRequestH\x00R\x0etestConnection\x12K\n\x10invoke_procedure\x18\x07 \x01(\x0b2\x1e.bdk.v1.InvokeProcedureRequestH\x00R\x0finvokeProcedure\x12T\n\x13discover_procedures\x18\x08 \x01(\x0b2!.bdk.v1.DiscoverProceduresRequestH\x00R\x12discoverProcedures\x12]\n\x16retrieve_discoverables\x18\t \x01(\x0b2$.bdk.v1.RetrieveDiscoverablesRequestH\x00R\x15retrieveDiscoverables\x12B\n\rretrieve_tags\x18\n \x01(\x0b2\x1b.bdk.v1.RetrieveTagsRequestH\x00R\x0cretrieveTags\x12H\n\x0fresolve_promise\x18\x0b \x01(\x0b2\x1d.bdk.v1.ResolvePromiseRequestH\x00R\x0eresolvePromiseB\x17\n\x15request_discriminatorB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.requests.request_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_REQUEST']._serialized_start = 487
    _globals['_REQUEST']._serialized_end = 1365


from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Optional as _Optional
DESCRIPTOR: _descriptor.FileDescriptor

class RetrieveTagsResponse(_message.Message):
    __slots__ = ('tags',)
    TAGS_FIELD_NUMBER: _ClassVar[int]
    tags: _containers.RepeatedScalarFieldContainer[str]

    def __init__(self, tags: _Optional[_Iterable[str]]=...) -> None:
        ...


from bdk.v2.types import value_pb2 as _value_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class Authentication(_message.Message):
    __slots__ = ('authentication_id', 'authentication_credentials')
    AUTHENTICATION_ID_FIELD_NUMBER: _ClassVar[int]
    AUTHENTICATION_CREDENTIALS_FIELD_NUMBER: _ClassVar[int]
    authentication_id: str
    authentication_credentials: _containers.RepeatedCompositeFieldContainer[CredentialValue]

    def __init__(self, authentication_id: _Optional[str]=..., authentication_credentials: _Optional[_Iterable[_Union[CredentialValue, _Mapping]]]=...) -> None:
        ...

class CredentialValue(_message.Message):
    __slots__ = ('id', 'value')
    ID_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    id: str
    value: _value_pb2.Value

    def __init__(self, id: _Optional[str]=..., value: _Optional[_Union[_value_pb2.Value, _Mapping]]=...) -> None:
        ...


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.bdk/v1/requests/retrieve_book_procedures.proto\x12\x06bdk.v1"v\n\x1dRetrieveBookProceduresRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x18\n\x07version\x18\x02 \x01(\tR\x07version\x12\'\n\x0finclude_connect\x18\x03 \x01(\x08R\x0eincludeConnectB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.requests.retrieve_book_procedures_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_RETRIEVEBOOKPROCEDURESREQUEST']._serialized_start = 58
    _globals['_RETRIEVEBOOKPROCEDURESREQUEST']._serialized_end = 176


from dataclasses import dataclass, field
from typing import List

from .base import BdkDescriptorBase
from .book_authentication_descriptor import BookAuthenticationDescriptor
from .concept_descriptor import ConceptDescriptor
from .noun_phrase import NounPhrase


@dataclass
class BookDescriptor(BdkDescriptorBase):
    id: str
    name: str
    short_description: str
    long_description: str
    author: str
    icon: bytes
    version: str
    authentications: List[BookAuthenticationDescriptor]
    configurations: List[ConceptDescriptor]
    display_name: str
    endpoint: str
    connection_required: bool
    discover_capable: bool
    noun_phrase: NounPhrase
    tags: List[str] = field(default_factory=list)



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional
DESCRIPTOR: _descriptor.FileDescriptor

class Label(_message.Message):
    __slots__ = ('key', 'value')
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str

    def __init__(self, key: _Optional[str]=..., value: _Optional[str]=...) -> None:
        ...


from bdk.v1.types import oauth_argument_descriptor_pb2 as _oauth_argument_descriptor_pb2
from bdk.v1.types import oauth_flow_pb2 as _oauth_flow_pb2
from bdk.v1.types import oauth_provider_pb2 as _oauth_provider_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class BookOAuthAuthenticationDescriptor(_message.Message):
    __slots__ = ('id', 'provider', 'flows', 'authorize_endpoint', 'token_endpoint', 'scope', 'name', 'arguments')
    ID_FIELD_NUMBER: _ClassVar[int]
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    FLOWS_FIELD_NUMBER: _ClassVar[int]
    AUTHORIZE_ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    TOKEN_ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    SCOPE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    ARGUMENTS_FIELD_NUMBER: _ClassVar[int]
    id: str
    provider: _oauth_provider_pb2.OAuthProvider
    flows: _containers.RepeatedScalarFieldContainer[_oauth_flow_pb2.OAuthFlow]
    authorize_endpoint: str
    token_endpoint: str
    scope: _containers.RepeatedScalarFieldContainer[str]
    name: str
    arguments: _containers.RepeatedCompositeFieldContainer[_oauth_argument_descriptor_pb2.OAuthArgumentDescriptor]

    def __init__(self, id: _Optional[str]=..., provider: _Optional[_Union[_oauth_provider_pb2.OAuthProvider, str]]=..., flows: _Optional[_Iterable[_Union[_oauth_flow_pb2.OAuthFlow, str]]]=..., authorize_endpoint: _Optional[str]=..., token_endpoint: _Optional[str]=..., scope: _Optional[_Iterable[str]]=..., name: _Optional[str]=..., arguments: _Optional[_Iterable[_Union[_oauth_argument_descriptor_pb2.OAuthArgumentDescriptor, _Mapping]]]=...) -> None:
        ...


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v2.types import book_connection_descriptor_pb2 as bdk_dot_v2_dot_types_dot_book__connection__descriptor__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&bdk/v2/responses/book_connection.proto\x12\x06bdk.v2\x1a-bdk/v2/types/book_connection_descriptor.proto"\xea\x01\n\x16BookConnectionResponse\x12`\n\x1abook_connection_descriptor\x18\x01 \x01(\x0b2 .bdk.v2.BookConnectionDescriptorH\x00R\x18bookConnectionDescriptor\x12T\n\x0eoauth_redirect\x18\x02 \x01(\x0b2+.bdk.v2.BookConnectionOAuthRedirectResponseH\x00R\roauthRedirectB\x18\n\x16response_discriminator"7\n#BookConnectionOAuthRedirectResponse\x12\x10\n\x03url\x18\x01 \x01(\tR\x03urlB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v2.responses.book_connection_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_BOOKCONNECTIONRESPONSE']._serialized_start = 98
    _globals['_BOOKCONNECTIONRESPONSE']._serialized_end = 332
    _globals['_BOOKCONNECTIONOAUTHREDIRECTRESPONSE']._serialized_start = 334
    _globals['_BOOKCONNECTIONOAUTHREDIRECTRESPONSE']._serialized_end = 389


import re
from dataclasses import dataclass
from typing import List, Optional

from .base import BdkDescriptorBase


@dataclass
class NounPhrase(BdkDescriptorBase):
    head: str
    modifiers: Optional[List[str]] = None

    def __post_init__(self):
        if self.modifiers is None:
            self.modifiers = []

        super().__post_init__()

    def __str__(self) -> str:
        modifiers = self.modifiers or []
        return " ".join(modifiers + [self.head])

    def __hash__(self) -> int:
        return hash(str(self))

    @staticmethod
    def from_str(s: str):
        """
        Creates a NounPhrase from a string. The string must be in the format
        <modifier1> <modifier2> ... <head>
        """
        *modifiers, head = s.split(" ")
        return NounPhrase(head=head, modifiers=modifiers)

    @classmethod
    def from_snake_case(cls, snake_str: str) -> "NounPhrase":
        """
        Creates a NounPhrase from a snake_case string.
        """
        # replace underscores with spaces
        words = snake_str.replace("_", " ")
        return NounPhrase(head=words, modifiers=[])

    @classmethod
    def from_pascal_case(cls, pascal_str: str) -> "NounPhrase":
        """
        Creates a NounPhrase from a pascal_case string.
        """
        # remove existing spaces
        pascal_str = pascal_str.replace(" ", "")
        # insert space before each capital letter (except the first one)
        words = re.sub(
            r"(?<=[a-z])(?=[A-Z])|(?<=[A-Z])(?=[A-Z][a-z])", " ", pascal_str
        ).lower()
        return NounPhrase(head=words, modifiers=[])

    def to_camel_case(self) -> str:
        """
        Converts the NounPhrase to camelCase.
        """
        snake_str = self.to_snake_case()
        components = snake_str.split("_")
        return components[0] + "".join(x.title() for x in components[1:])

    def to_snake_case(self) -> str:
        """
        Converts the NounPhrase to snake_case.
        """
        text = str(self)
        return text.lower().replace(" ", "_")

    def to_kebab_case(self) -> str:
        """
        Converts the NounPhrase to kebab-case.
        """
        snake_str = self.to_snake_case()
        return snake_str.replace("_", "-")


@dataclass
class NounPhrases(BdkDescriptorBase):
    noun_phrases: List[NounPhrase]

    def __str__(self) -> str:
        return "'s ".join([str(noun_phrase) for noun_phrase in self.noun_phrases])

    def __hash__(self) -> int:
        return hash(str(self))

    @staticmethod
    def from_str(s: str):
        """
        Creates a NounPhrases from a string. The string must be in the format
        <modifier1> <modifier2> ... <head>'s <modifier1> <modifier2> ... <head>'s ...
        """
        noun_phrases = []
        for np in s.split("'s"):
            if np:
                noun_phrases.append(NounPhrase.from_str(np))
        return NounPhrases(noun_phrases=noun_phrases)



from typing import List

# pylint: disable=no-name-in-module
from kognitos.bdk.runtime.client.proto.bdk.v1.responses.retrieve_discoverables_pb2 import \
    RetrieveDiscoverablesResponse as ProtoDiscoverablesResponse

from ..discoverable import Discoverable


def map_discoverables(
    data: ProtoDiscoverablesResponse,
) -> List[Discoverable]:
    return [
        Discoverable(name=d.name, description=d.description) for d in data.discoverables
    ]



from dataclasses import dataclass
from enum import Enum

from .base import BdkDescriptorBase


class CredentialType(Enum):
    CREDENTIAL_TYPE_TEXT = 0
    CREDENTIAL_TYPE_SENSITIVE_TEXT = 1


@dataclass
class CredentialDescriptor(BdkDescriptorBase):
    id: str
    type: CredentialType
    label: str
    visible: bool
    description: str



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional
DESCRIPTOR: _descriptor.FileDescriptor

class Time(_message.Message):
    __slots__ = ('hours', 'minutes', 'seconds', 'nanos')
    HOURS_FIELD_NUMBER: _ClassVar[int]
    MINUTES_FIELD_NUMBER: _ClassVar[int]
    SECONDS_FIELD_NUMBER: _ClassVar[int]
    NANOS_FIELD_NUMBER: _ClassVar[int]
    hours: int
    minutes: int
    seconds: int
    nanos: int

    def __init__(self, hours: _Optional[int]=..., minutes: _Optional[int]=..., seconds: _Optional[int]=..., nanos: _Optional[int]=...) -> None:
        ...


# pylint: disable=no-name-in-module
from kognitos.bdk.runtime.client.proto.bdk.v1.types.book_procedure_descriptor_pb2 import \
    BookProcedureDescriptor as ProtoBookProcedureDescriptor
from kognitos.bdk.runtime.client.proto.bdk.v1.types.book_procedure_descriptor_pb2 import \
    BookProcedureDescriptorV2 as ProtoBookProcedureDescriptorV2
from kognitos.bdk.runtime.client.proto.bdk.v1.types.connection_required_pb2 import \
    Optional as ConnectionRequiredOptional

# pylint: enable=no-name-in-module
from ..book_procedure_descriptor import BookProcedureDescriptor
from ..connection_required import ConnectionRequired
from .book_procedure_signature import map_book_procedure_signature
from .concept_descriptor import map_concept_descriptor
from .example_descriptor import map_example_descriptor
from .question_descriptor import map_question_descriptor


def map_book_procedure_descriptor(
    data: ProtoBookProcedureDescriptor,
) -> BookProcedureDescriptor:
    if (
        data.connection_requirement_level == ConnectionRequiredOptional
        and data.connection_required
    ):
        connection_required = ConnectionRequired.ALWAYS
    else:
        connection_required = ConnectionRequired(data.connection_requirement_level)

    return BookProcedureDescriptor(
        id=data.id,
        short_description=data.short_description,
        long_description=data.long_description,
        signature=map_book_procedure_signature(data.signature),
        inputs=[map_concept_descriptor(cd) for cd in data.inputs],
        outputs=[map_concept_descriptor(cd) for cd in data.outputs],
        questions=[map_question_descriptor(qd) for qd in data.questions],
        filter_capable=data.filter_capable,
        page_capable=data.page_capable,
        connection_required=connection_required,
        examples=[map_example_descriptor(example) for example in data.examples],
    )


def map_book_procedure_descriptor_v2(
    data: ProtoBookProcedureDescriptorV2,
) -> BookProcedureDescriptor:
    return BookProcedureDescriptor(
        id=data.id,
        short_description=data.short_description,
        long_description=data.long_description,
        signature=map_book_procedure_signature(data.signature),
        inputs=[map_concept_descriptor(cd) for cd in data.inputs],
        outputs=[map_concept_descriptor(cd) for cd in data.outputs],
        questions=[map_question_descriptor(qd) for qd in data.questions],
        filter_capable=data.filter_capable,
        page_capable=data.page_capable,
        connection_required=ConnectionRequired(data.connection_required),
        examples=[map_example_descriptor(example) for example in data.examples],
    )



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cbdk/v2/requests/labels.proto\x12\x06bdk.v2"/\n\x05Label\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05valueB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v2.requests.labels_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_LABEL']._serialized_start = 40
    _globals['_LABEL']._serialized_end = 87


# pylint: disable=no-name-in-module
from kognitos.bdk.runtime.client.example_descriptor import ExampleDescriptor
from kognitos.bdk.runtime.client.proto.bdk.v1.types.example_descriptor_pb2 import \
    ExampleDescriptor as ProtoExampleDescriptor


def map_example_descriptor(data: ProtoExampleDescriptor) -> ExampleDescriptor:
    return ExampleDescriptor(description=data.description, snippet=data.snippet)



from bdk.v1 import v1_pb2 as _v1_pb2
from bdk.v2 import v2_pb2 as _v2_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from grpc.health.v1 import health_pb2 as _health_pb2
import version_pb2 as _version_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar
DESCRIPTOR: _descriptor.FileDescriptor


from dataclasses import dataclass
from typing import List, Optional

from .base import BdkDescriptorBase
from .concept_type import ConceptType
from .noun_phrase import NounPhrases
from .value import Value


@dataclass
class Question(BdkDescriptorBase):
    noun_phrases: NounPhrases
    concept_type: ConceptType
    choices: Optional[List[Value]] = None
    text: Optional[str] = None


@dataclass
class AnsweredQuestion(BdkDescriptorBase):
    noun_phrases: NounPhrases
    answer: Value



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v2.types import concept_type_pb2 as bdk_dot_v2_dot_types_dot_concept__type__pb2
from ....bdk.v2.types import date_pb2 as bdk_dot_v2_dot_types_dot_date__pb2
from ....bdk.v2.types import noun_phrase_pb2 as bdk_dot_v2_dot_types_dot_noun__phrase__pb2
from ....bdk.v2.types import time_pb2 as bdk_dot_v2_dot_types_dot_time__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18bdk/v2/types/value.proto\x12\x06bdk.v2\x1a\x1fbdk/v2/types/concept_type.proto\x1a\x17bdk/v2/types/date.proto\x1a\x1ebdk/v2/types/noun_phrase.proto\x1a\x17bdk/v2/types/time.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x8e\x06\n\x05Value\x12;\n\nnull_value\x18\x01 \x01(\x0e2\x1a.google.protobuf.NullValueH\x00R\tnullValue\x12?\n\x10conceptual_value\x18\x02 \x01(\x0b2\x12.bdk.v2.NounPhraseH\x00R\x0fconceptualValue\x12\x1f\n\ntext_value\x18\x03 \x01(\tH\x00R\ttextValue\x12#\n\x0cnumber_value\x18\x04 \x01(\x01H\x00R\x0bnumberValue\x12%\n\rboolean_value\x18\x05 \x01(\x08H\x00R\x0cbooleanValue\x12C\n\x0edatetime_value\x18\x06 \x01(\x0b2\x1a.google.protobuf.TimestampH\x00R\rdatetimeValue\x12-\n\ndate_value\x18\x07 \x01(\x0b2\x0c.bdk.v2.DateH\x00R\tdateValue\x12-\n\ntime_value\x18\x08 \x01(\x0b2\x0c.bdk.v2.TimeH\x00R\ttimeValue\x122\n\nfile_value\x18\t \x01(\x0b2\x11.bdk.v2.FileValueH\x00R\tfileValue\x12D\n\x10dictionary_value\x18\n \x01(\x0b2\x17.bdk.v2.DictionaryValueH\x00R\x0fdictionaryValue\x122\n\nlist_value\x18\x0b \x01(\x0b2\x11.bdk.v2.ListValueH\x00R\tlistValue\x128\n\x0copaque_value\x18\x0c \x01(\x0b2\x13.bdk.v2.OpaqueValueH\x00R\x0bopaqueValue\x125\n\x0btable_value\x18\r \x01(\x0b2\x12.bdk.v2.TableValueH\x00R\ntableValue\x12A\n\x0fsensitive_value\x18\x0e \x01(\x0b2\x16.bdk.v2.SensitiveValueH\x00R\x0esensitiveValueB\x15\n\x13value_discriminator"n\n\x0fDictionaryValue\x124\n\x06fields\x18\x01 \x03(\x0b2\x1c.bdk.v2.DictionaryValueFieldR\x06fields\x12%\n\x04is_a\x18\x02 \x03(\x0b2\x12.bdk.v2.NounPhraseR\x03isA"M\n\x14DictionaryValueField\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12#\n\x05value\x18\x02 \x01(\x0b2\r.bdk.v2.ValueR\x05value"2\n\tListValue\x12%\n\x06values\x18\x01 \x03(\x0b2\r.bdk.v2.ValueR\x06values"^\n\tFileValue\x12\x18\n\x06remote\x18\x01 \x01(\tH\x00R\x06remote\x12&\n\x06inline\x18\x02 \x01(\x0b2\x0c.bdk.v2.FileH\x00R\x06inlineB\x0f\n\rinline_remote"=\n\x04File\x12\x1b\n\tfile_name\x18\x01 \x01(\tR\x08fileName\x12\x18\n\x07content\x18\x02 \x01(\x0cR\x07content"N\n\x0bOpaqueValue\x12\x18\n\x07content\x18\x01 \x01(\x0cR\x07content\x12%\n\x04is_a\x18\x02 \x03(\x0b2\x12.bdk.v2.NounPhraseR\x03isA"5\n\x0eSensitiveValue\x12#\n\x05value\x18\x01 \x01(\x0b2\r.bdk.v2.ValueR\x05value"`\n\nTableValue\x12\x18\n\x06remote\x18\x01 \x01(\tH\x00R\x06remote\x12\'\n\x06inline\x18\x02 \x01(\x0b2\r.bdk.v2.TableH\x00R\x06inlineB\x0f\n\rinline_remote"1\n\x05Table\x12(\n\x07columns\x18\x01 \x03(\x0b2\x0e.bdk.v2.ColumnR\x07columns"C\n\x06Column\x12%\n\x06values\x18\x01 \x03(\x0b2\r.bdk.v2.ValueR\x06values\x12\x12\n\x04name\x18\x02 \x01(\tR\x04nameB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v2.types.value_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_VALUE']._serialized_start = 215
    _globals['_VALUE']._serialized_end = 997
    _globals['_DICTIONARYVALUE']._serialized_start = 999
    _globals['_DICTIONARYVALUE']._serialized_end = 1109
    _globals['_DICTIONARYVALUEFIELD']._serialized_start = 1111
    _globals['_DICTIONARYVALUEFIELD']._serialized_end = 1188
    _globals['_LISTVALUE']._serialized_start = 1190
    _globals['_LISTVALUE']._serialized_end = 1240
    _globals['_FILEVALUE']._serialized_start = 1242
    _globals['_FILEVALUE']._serialized_end = 1336
    _globals['_FILE']._serialized_start = 1338
    _globals['_FILE']._serialized_end = 1399
    _globals['_OPAQUEVALUE']._serialized_start = 1401
    _globals['_OPAQUEVALUE']._serialized_end = 1479
    _globals['_SENSITIVEVALUE']._serialized_start = 1481
    _globals['_SENSITIVEVALUE']._serialized_end = 1534
    _globals['_TABLEVALUE']._serialized_start = 1536
    _globals['_TABLEVALUE']._serialized_end = 1632
    _globals['_TABLE']._serialized_start = 1634
    _globals['_TABLE']._serialized_end = 1683
    _globals['_COLUMN']._serialized_start = 1685
    _globals['_COLUMN']._serialized_end = 1752


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v1.requests import authentication_pb2 as bdk_dot_v1_dot_requests_dot_authentication__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,bdk/v1/requests/retrieve_discoverables.proto\x12\x06bdk.v1\x1a$bdk/v1/requests/authentication.proto"\x81\x02\n\x1cRetrieveDiscoverablesRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x18\n\x07version\x18\x02 \x01(\tR\x07version\x12>\n\x0eauthentication\x18\x03 \x01(\x0b2\x16.bdk.v1.AuthenticationR\x0eauthentication\x12\x1b\n\x06search\x18\x04 \x01(\tH\x00R\x06search\x88\x01\x01\x12\x1b\n\x06offset\x18\x05 \x01(\x04H\x01R\x06offset\x88\x01\x01\x12\x19\n\x05limit\x18\x06 \x01(\x04H\x02R\x05limit\x88\x01\x01B\t\n\x07_searchB\t\n\x07_offsetB\x08\n\x06_limitB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.requests.retrieve_discoverables_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_RETRIEVEDISCOVERABLESREQUEST']._serialized_start = 95
    _globals['_RETRIEVEDISCOVERABLESREQUEST']._serialized_end = 352


from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class ErrorKind(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    ErrorKindInternal: _ClassVar[ErrorKind]
    ErrorKindNotFound: _ClassVar[ErrorKind]
    ErrorKindNotSupported: _ClassVar[ErrorKind]
    ErrorKindMissing: _ClassVar[ErrorKind]
    ErrorKindInvalidValue: _ClassVar[ErrorKind]
    ErrorKindHTTP: _ClassVar[ErrorKind]
    ErrorKindRateLimited: _ClassVar[ErrorKind]
    ErrorKindAccessDenied: _ClassVar[ErrorKind]
    ErrorKindTypeMismatch: _ClassVar[ErrorKind]
    ErrorKindTLS: _ClassVar[ErrorKind]
    ErrorKindAuthenticationRequired: _ClassVar[ErrorKind]
    ErrorKindDeserializationError: _ClassVar[ErrorKind]
ErrorKindInternal: ErrorKind
ErrorKindNotFound: ErrorKind
ErrorKindNotSupported: ErrorKind
ErrorKindMissing: ErrorKind
ErrorKindInvalidValue: ErrorKind
ErrorKindHTTP: ErrorKind
ErrorKindRateLimited: ErrorKind
ErrorKindAccessDenied: ErrorKind
ErrorKindTypeMismatch: ErrorKind
ErrorKindTLS: ErrorKind
ErrorKindAuthenticationRequired: ErrorKind
ErrorKindDeserializationError: ErrorKind

class Error(_message.Message):
    __slots__ = ('kind', 'message', 'extra')

    class ExtraEntry(_message.Message):
        __slots__ = ('key', 'value')
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str

        def __init__(self, key: _Optional[str]=..., value: _Optional[str]=...) -> None:
            ...
    KIND_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    EXTRA_FIELD_NUMBER: _ClassVar[int]
    kind: ErrorKind
    message: str
    extra: _containers.ScalarMap[str, str]

    def __init__(self, kind: _Optional[_Union[ErrorKind, str]]=..., message: _Optional[str]=..., extra: _Optional[_Mapping[str, str]]=...) -> None:
        ...


from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class Offload(_message.Message):
    __slots__ = ('aws',)
    AWS_FIELD_NUMBER: _ClassVar[int]
    aws: AWSOffload

    def __init__(self, aws: _Optional[_Union[AWSOffload, _Mapping]]=...) -> None:
        ...

class AWSOffload(_message.Message):
    __slots__ = ('access_key', 'secret_key', 'session_token', 'region', 'bucket', 'folder_name', 'endpoint_url')
    ACCESS_KEY_FIELD_NUMBER: _ClassVar[int]
    SECRET_KEY_FIELD_NUMBER: _ClassVar[int]
    SESSION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    REGION_FIELD_NUMBER: _ClassVar[int]
    BUCKET_FIELD_NUMBER: _ClassVar[int]
    FOLDER_NAME_FIELD_NUMBER: _ClassVar[int]
    ENDPOINT_URL_FIELD_NUMBER: _ClassVar[int]
    access_key: str
    secret_key: str
    session_token: str
    region: str
    bucket: str
    folder_name: str
    endpoint_url: str

    def __init__(self, access_key: _Optional[str]=..., secret_key: _Optional[str]=..., session_token: _Optional[str]=..., region: _Optional[str]=..., bucket: _Optional[str]=..., folder_name: _Optional[str]=..., endpoint_url: _Optional[str]=...) -> None:
        ...


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v2.types import value_pb2 as bdk_dot_v2_dot_types_dot_value__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1abdk/v2/types/promise.proto\x12\x06bdk.v2\x1a\x18bdk/v2/types/value.proto"q\n\x07Promise\x12C\n\x1epromise_resolver_function_name\x18\x01 \x01(\tR\x1bpromiseResolverFunctionName\x12!\n\x04data\x18\x02 \x01(\x0b2\r.bdk.v2.ValueR\x04dataB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v2.types.promise_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_PROMISE']._serialized_start = 64
    _globals['_PROMISE']._serialized_end = 177


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v1.responses import promise_pb2 as bdk_dot_v1_dot_responses_dot_promise__pb2
from ....bdk.v1.responses import question_pb2 as bdk_dot_v1_dot_responses_dot_question__pb2
from ....bdk.v1.types import concept_value_pb2 as bdk_dot_v1_dot_types_dot_concept__value__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'bdk/v1/responses/invoke_procedure.proto\x12\x06bdk.v1\x1a\x1ebdk/v1/responses/promise.proto\x1a\x1fbdk/v1/responses/question.proto\x1a bdk/v1/types/concept_value.proto"X\n\x17InvokeProcedureResponse\x12=\n\x0foutput_concepts\x18\x01 \x03(\x0b2\x14.bdk.v1.ConceptValueR\x0eoutputConcepts"\xe1\x01\n\x19InvokeProcedureResponseV2\x12=\n\x08response\x18\x01 \x01(\x0b2\x1f.bdk.v1.InvokeProcedureResponseH\x00R\x08response\x126\n\x08question\x18\x02 \x01(\x0b2\x18.bdk.v1.QuestionResponseH\x00R\x08question\x123\n\x07promise\x18\x03 \x01(\x0b2\x17.bdk.v1.PromiseResponseH\x00R\x07promiseB\x18\n\x16response_discriminatorB\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v1.responses.invoke_procedure_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_INVOKEPROCEDURERESPONSE']._serialized_start = 150
    _globals['_INVOKEPROCEDURERESPONSE']._serialized_end = 238
    _globals['_INVOKEPROCEDURERESPONSEV2']._serialized_start = 241
    _globals['_INVOKEPROCEDURERESPONSEV2']._serialized_end = 466


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_sym_db = _symbol_database.Default()
from ....bdk.v2.requests import discover_procedures_pb2 as bdk_dot_v2_dot_requests_dot_discover__procedures__pb2
from ....bdk.v2.requests import environment_information_pb2 as bdk_dot_v2_dot_requests_dot_environment__information__pb2
from ....bdk.v2.requests import invoke_procedure_pb2 as bdk_dot_v2_dot_requests_dot_invoke__procedure__pb2
from ....bdk.v2.requests import resolve_promise_pb2 as bdk_dot_v2_dot_requests_dot_resolve__promise__pb2
from ....bdk.v2.requests import retrieve_book_pb2 as bdk_dot_v2_dot_requests_dot_retrieve__book__pb2
from ....bdk.v2.requests import retrieve_book_procedures_pb2 as bdk_dot_v2_dot_requests_dot_retrieve__book__procedures__pb2
from ....bdk.v2.requests import retrieve_books_pb2 as bdk_dot_v2_dot_requests_dot_retrieve__books__pb2
from ....bdk.v2.requests import retrieve_discoverables_pb2 as bdk_dot_v2_dot_requests_dot_retrieve__discoverables__pb2
from ....bdk.v2.requests import test_connection_pb2 as bdk_dot_v2_dot_requests_dot_test__connection__pb2
from ....bdk.v2.responses import discover_procedures_pb2 as bdk_dot_v2_dot_responses_dot_discover__procedures__pb2
from ....bdk.v2.responses import environment_information_pb2 as bdk_dot_v2_dot_responses_dot_environment__information__pb2
from ....bdk.v2.responses import invoke_procedure_pb2 as bdk_dot_v2_dot_responses_dot_invoke__procedure__pb2
from ....bdk.v2.responses import retrieve_book_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__book__pb2
from ....bdk.v2.responses import retrieve_book_procedures_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__book__procedures__pb2
from ....bdk.v2.responses import retrieve_books_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__books__pb2
from ....bdk.v2.responses import retrieve_discoverables_pb2 as bdk_dot_v2_dot_responses_dot_retrieve__discoverables__pb2
from ....bdk.v2.responses import test_connection_pb2 as bdk_dot_v2_dot_responses_dot_test__connection__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1abdk/v2/services/book.proto\x12\x06bdk.v2\x1a)bdk/v2/requests/discover_procedures.proto\x1a-bdk/v2/requests/environment_information.proto\x1a&bdk/v2/requests/invoke_procedure.proto\x1a%bdk/v2/requests/resolve_promise.proto\x1a#bdk/v2/requests/retrieve_book.proto\x1a.bdk/v2/requests/retrieve_book_procedures.proto\x1a$bdk/v2/requests/retrieve_books.proto\x1a,bdk/v2/requests/retrieve_discoverables.proto\x1a%bdk/v2/requests/test_connection.proto\x1a*bdk/v2/responses/discover_procedures.proto\x1a.bdk/v2/responses/environment_information.proto\x1a\'bdk/v2/responses/invoke_procedure.proto\x1a$bdk/v2/responses/retrieve_book.proto\x1a/bdk/v2/responses/retrieve_book_procedures.proto\x1a%bdk/v2/responses/retrieve_books.proto\x1a-bdk/v2/responses/retrieve_discoverables.proto\x1a&bdk/v2/responses/test_connection.proto2\xb8\x06\n\x0bBookService\x12g\n\x16EnvironmentInformation\x12%.bdk.v2.EnvironmentInformationRequest\x1a&.bdk.v2.EnvironmentInformationResponse\x12L\n\rRetrieveBooks\x12\x1c.bdk.v2.RetrieveBooksRequest\x1a\x1d.bdk.v2.RetrieveBooksResponse\x12I\n\x0cRetrieveBook\x12\x1b.bdk.v2.RetrieveBookRequest\x1a\x1c.bdk.v2.RetrieveBookResponse\x12i\n\x16RetrieveBookProcedures\x12%.bdk.v2.RetrieveBookProceduresRequest\x1a(.bdk.v2.RetrieveBookProceduresResponseV2\x12O\n\x0eTestConnection\x12\x1d.bdk.v2.TestConnectionRequest\x1a\x1e.bdk.v2.TestConnectionResponse\x12T\n\x0fInvokeProcedure\x12\x1e.bdk.v2.InvokeProcedureRequest\x1a!.bdk.v2.InvokeProcedureResponseV2\x12[\n\x12DiscoverProcedures\x12!.bdk.v2.DiscoverProceduresRequest\x1a".bdk.v2.DiscoverProceduresResponse\x12d\n\x15RetrieveDiscoverables\x12$.bdk.v2.RetrieveDiscoverablesRequest\x1a%.bdk.v2.RetrieveDiscoverablesResponse\x12R\n\x0eResolvePromise\x12\x1d.bdk.v2.ResolvePromiseRequest\x1a!.bdk.v2.InvokeProcedureResponseV2B\x1fZ\x1dgithub.com/kognitos/bdk-protob\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bdk.v2.services.book_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals['DESCRIPTOR']._options = None
    _globals['DESCRIPTOR']._serialized_options = b'Z\x1dgithub.com/kognitos/bdk-proto'
    _globals['_BOOKSERVICE']._serialized_start = 762
    _globals['_BOOKSERVICE']._serialized_end = 1586


from bdk.v1.types import value_pb2 as _value_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class Promise(_message.Message):
    __slots__ = ('promise_resolver_function_name', 'data')
    PROMISE_RESOLVER_FUNCTION_NAME_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    promise_resolver_function_name: str
    data: _value_pb2.Value

    def __init__(self, promise_resolver_function_name: _Optional[str]=..., data: _Optional[_Union[_value_pb2.Value, _Mapping]]=...) -> None:
        ...


from bdk.v1.types import credential_type_pb2 as _credential_type_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union
DESCRIPTOR: _descriptor.FileDescriptor

class CredentialDescriptor(_message.Message):
    __slots__ = ('id', 'type', 'label', 'visible', 'description')
    ID_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    LABEL_FIELD_NUMBER: _ClassVar[int]
    VISIBLE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    id: str
    type: _credential_type_pb2.CredentialType
    label: str
    visible: bool
    description: str

    def __init__(self, id: _Optional[str]=..., type: _Optional[_Union[_credential_type_pb2.CredentialType, str]]=..., label: _Optional[str]=..., visible: bool=..., description: _Optional[str]=...) -> None:
        ...